@@ -23,11 +23,14 @@
 #
 #
 # Author: Ilya Baldin (ibaldin@renci.org)
-from typing import List, Dict
+from typing import List, Dict, Tuple
 import json
 
 from abc import ABC, abstractmethod
 
+import requests
+import urllib.parse
+
 from fim.graph.abc_property_graph_constants import ABCPropertyGraphConstants
 
 
@@ -56,18 +59,19 @@ class JSONField(ABC):
             return ''
         return json.dumps(d, skipkeys=True, sort_keys=True)
 
-    def from_json(self, json_string: str):
+    @classmethod
+    def from_json(cls, json_string: str):
         """
-        Set fields from json string (if string is none or empty,
-        nothing happens)
+        Set fields from json string and returns a new object
         :param json_string:
-        :return: self for call chaining
+        :return: object
         """
         if json_string is None or len(json_string) == 0 or json_string == ABCPropertyGraphConstants.NEO4j_NONE:
-            return self
+            return None
         d = json.loads(json_string)
-        self.set_fields(**d)
-        return self
+        ret = cls()
+        ret.set_fields(**d)
+        return ret
 
     def to_dict(self) -> Dict[str, str] or None:
         """
@@ -100,15 +104,17 @@ class Capacities(JSONField):
     Implements basic capacity field handling - encoding and decoding
     from JSON dictionaries of properties
     """
-    UNITS = {'cpu': '', 'unit': '', 'core': '', 'ram': 'G', 'disk': 'G', 'bw': 'G'}
+    UNITS = {'cpu': '', 'unit': '', 'core': '', 'ram': 'G', 'disk': 'G', 'bw': 'Gbps', 'burst_size': 'Mbits'}
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         self.cpu = 0
         self.core = 0
         self.ram = 0
         self.disk = 0
         self.bw = 0
+        self.burst_size = 0
         self.unit = 0
+        self.set_fields(**kwargs)
 
     def set_fields(self, **kwargs):
         """
@@ -126,7 +132,8 @@ class Capacities(JSONField):
                 self.__getattribute__(k)
                 self.__setattr__(k, v)
             except AttributeError:
-                raise CapacityException(f"Unable to set field {k} of capacity, no such field available")
+                raise CapacityException(f"Unable to set field {k} of capacity, no such field available "
+                                        f"{[k for k in self.__dict__.keys()]}")
         return self
 
     def __add__(self, other):
@@ -146,6 +153,27 @@ class Capacities(JSONField):
 
         return ret
 
+    def __gt__(self, other):
+        assert isinstance(other, Capacities)
+        for f, v in self.__dict__.items():
+            if v < other.__dict__[f]:
+                return False
+        return True
+
+    def __lt__(self, other):
+        assert isinstance(other, Capacities)
+        for f, v in self.__dict__.items():
+            if v > other.__dict__[f]:
+                return False
+        return True
+
+    def __eq__(self, other):
+        assert isinstance(other, Capacities)
+        for f, v in self.__dict__.items():
+            if v != other.__dict__[f]:
+                return False
+        return True
+
     def negative_fields(self) -> List[str]:
         """
         returns list of fields that are negative
@@ -179,8 +207,8 @@ class Capacities(JSONField):
             return ''
         ret = "{ "
         for i, v in d.items():
-            ret = ret + i + ": " + str(v) + self.UNITS[i] + ", "
-        return ret + "}"
+            ret = ret + i + ": " + f'{v:,} ' + self.UNITS[i] + ", "
+        return ret[:-2] + "}"
 
 
 class CapacityTuple:
@@ -207,8 +235,54 @@ class CapacityTuple:
             return ''
         ret = '{ '
         for k in d1:
-            ret = ret + k + ": " + str(d1[k]) + "/" + str(d2[k]) + Capacities.UNITS[k] + ", "
-        return ret + '}'
+            ret = ret + k + ": " + f'{d1[k]:,}' + "/" + f'{d2[k]:,} ' + Capacities.UNITS[k] + ", "
+        return ret[:-2] + '}'
+
+
+class CapacityHints(JSONField):
+    """
+    Capacity hints are strings representing something about the capacity of a sliver,
+    e.g. an instance size name. They have to be mappable to actual capacities
+    via e.g. a catalog.
+    """
+    def __init__(self, **kwargs):
+        self.instance_type = None
+        self.set_fields(**kwargs)
+
+    def set_fields(self, **kwargs):
+        """
+        Universal setter for all fields. Values should be strings.
+        Throws a LabelException if you try to set a non-existent field.
+        :param kwargs:
+        :return: self to support call chaining
+        """
+        for k, v in kwargs.items():
+            assert v is not None  # could be strings
+            assert isinstance(v, str)
+            try:
+                # will toss an exception if field is not defined
+                self.__getattribute__(k)
+                self.__setattr__(k, v)
+            except AttributeError:
+                raise CapacityHintException(f"Unable to set field {k} of capacity hints, no such field available")
+        # to support call chaining
+        return self
+
+    def __add__(self, other):
+        assert isinstance(other, Labels)
+        raise RuntimeError("Not Implemented")
+
+    def __str__(self):
+        d = self.__dict__.copy()
+        for k in self.__dict__:
+            if d[k] is None:
+                d.pop(k)
+        if len(d) == 0:
+            return ''
+        ret = "{ "
+        for i, v in d.items():
+            ret = ret + i + ": " + str(v) + ", "
+        return ret[:-2] + "}"
 
 
 class Labels(JSONField):
@@ -216,7 +290,7 @@ class Labels(JSONField):
     Class implementing various encodings of labels field, encoding
     and decoding from JSON dictionaries of properties
     """
-    def __init__(self):
+    def __init__(self, **kwargs):
         self.bdf = None
         self.mac = None
         self.ipv4 = None
@@ -226,8 +300,13 @@ class Labels(JSONField):
         self.asn = None
         self.vlan = None
         self.vlan_range = None
+        self.inner_vlan = None
         self.instance = None
         self.instance_parent = None
+        self.local_name = None
+        self.local_type = None
+        self.device_name = None
+        self.set_fields(**kwargs)
 
     def set_fields(self, **kwargs):
         """
@@ -244,7 +323,8 @@ class Labels(JSONField):
                 self.__getattribute__(k)
                 self.__setattr__(k, v)
             except AttributeError:
-                raise LabelException(f"Unable to set field {k} of labels, no such field available")
+                raise LabelException(f"Unable to set field {k} of labels, no such field available "
+                                     f"{[k for k in self.__dict__.keys()]}")
         # to support call chaining
         return self
 
@@ -263,7 +343,7 @@ class Labels(JSONField):
         ret = "{ "
         for i, v in d.items():
             ret = ret + i + ": " + str(v) + ", "
-        return ret + "}"
+        return ret[:-2] + "}"
 
 
 class ReservationInfo(JSONField):
@@ -271,14 +351,14 @@ class ReservationInfo(JSONField):
     Reservation info structure for ASM sliver objects
     """
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         self.reservation_id = None
         self.reservation_state = None
+        self.set_fields(**kwargs)
 
     def set_fields(self, **kwargs):
         """
-        Universal setter for all fields (just strip the l_ from the field
-        name of the field). Values should be strings or lists of strings.
+        Universal setter for all fields. Values should be strings or lists of strings.
         Throws a ReservationInfoException if you try to set a non-existent field.
         :param kwargs:
         :return: self to support call chaining
@@ -291,14 +371,104 @@ class ReservationInfo(JSONField):
                 self.__getattribute__(k)
                 self.__setattr__(k, v)
             except AttributeError:
-                raise ReservationInfoException(f"Unable to set field {k} of reservation info, no such field available")
+                raise ReservationInfoException(f"Unable to set field {k} of reservation info, no such field "
+                                               f"available {[k for k in self.__dict__.keys()]}")
         # to support call chaining
         return self
 
 
+class StructuralInfo(JSONField):
+    """
+    Structural info on the for sliver objects (things like - is it a stitching node,
+    what is parent graph or subgraph)
+    """
+    def __init__(self):
+        self.sub_graph_id = None
+        self.parent_graph_id = None
+        self.adm_graph_ids = None
+
+    def set_fields(self, **kwargs):
+        """
+        Universal setter for all fields. Values are strings or boolean.
+        Throws a
+        :param kwargs:
+        :return:
+        """
+        for k, v in kwargs.items():
+            assert v is not None  # could be strings or lists of strings
+            assert isinstance(v, str) or isinstance(v, list)
+            try:
+                # will toss an exception if field is not defined
+                self.__getattribute__(k)
+                self.__setattr__(k, v)
+            except AttributeError:
+                raise StructuralInfoException(f"Unable to set field {k} of structural info, no such field available")
+        # to support call chaining
+        return self
+
+
+class Location(JSONField):
+    """
+    Location representation (as address, coordinates or any other useful way)
+    """
+    def __init__(self, **kwargs):
+        self.postal = None
+        self.set_fields(**kwargs)
+
+    def set_fields(self, **kwargs):
+        """
+        Universal setter for location fields. Values are strings.
+        :param kwargs:
+        :return:
+        """
+        for k, v in kwargs.items():
+            assert v is not None
+            assert isinstance(v, str)
+            try:
+                # will throw exception if field is not defined
+                self.__getattribute__(k)
+                self.__setattr__(k, v)
+            except AttributeError:
+                raise LocationException(f"Unable to set field {k} of location, no such field available")
+        return self
+
+    def to_latlon(self) -> Tuple[float, float]:
+        """
+        Return a tuple of floats indicating Lat/Lon of the location. Uses Nomatim OpenStreetMaps
+        service.
+        :return:
+        """
+        url = 'https://nominatim.openstreetmap.org/search/' + urllib.parse.quote(self.postal) + '?format=json'
+        # per terms of service set user agent
+        headers = {'User-Agent': 'FABRIC FIM Utility'}
+        response = requests.get(url, headers)
+        if response.status_code != 200:
+            raise LocationException(f"Unable to convert address to Lat/Lon via OpenStreetmaps due "
+                                    f"to: {response.reason}")
+        try:
+            response_json = response.json()
+            if not isinstance(response_json, list):
+                raise ValueError
+            if len(response_json) < 1:
+                raise ValueError
+        except ValueError:
+            raise LocationException(f"Unable to interpret response from OpenStreetmaps for address {self.postal}")
+
+        return float(response_json[0]['lat']), float(response_json[0]['lon'])
+
+
 class CapacityException(Exception):
     """
-    Exception with a pool
+    Exception with a capacity
+    """
+    def __init__(self, msg: str):
+        assert msg is not None
+        super().__init__(f"Delegation exception: {msg}")
+
+
+class CapacityHintException(Exception):
+    """
+    Exception with a capacity
     """
     def __init__(self, msg: str):
         assert msg is not None
@@ -307,7 +477,7 @@ class CapacityException(Exception):
 
 class LabelException(Exception):
     """
-    Exception with a pool
+    Exception with a label
     """
     def __init__(self, msg: str):
         assert msg is not None
@@ -316,8 +486,27 @@ class LabelException(Exception):
 
 class ReservationInfoException(Exception):
     """
-    Exception with a pool
+    Exception with a reservation info
     """
     def __init__(self, msg: str):
         assert msg is not None
-        super().__init__(f"ReservationInfo exception: {msg}")
\ No newline at end of file
+        super().__init__(f"ReservationInfo exception: {msg}")
+
+
+class StructuralInfoException(Exception):
+    """
+    Exception with a structural info
+    """
+
+    def __init__(self, msg: str):
+        assert msg is not None
+        super().__init__(f"ReservationInfo exception: {msg}")
+
+
+class LocationException(Exception):
+    """
+    Exception with location
+    """
+    def __init__(self, msg: str):
+        assert msg is not None
+        super().__init__(f"Location exception: {msg}")
