@@ -43,6 +43,13 @@ class NSLayer(enum.Enum):
     def __str__(self):
         return self.name
 
+    @classmethod
+    def from_string(cls, s: str):
+        for la in NSLayer:
+            if la.name == s:
+                return cls(la)
+        return None
+
 
 class ServiceType(enum.Enum):
     """
@@ -221,3 +228,6 @@ class NetworkServiceInfo:
     def get_network_service_names(self):
         return list(self.network_services.keys())
 
+    def list_services(self):
+        return list(self.network_services.values())
+
