@@ -29,6 +29,7 @@ from abc import ABC, abstractmethod
 import enum
 
 from ..graph.abc_property_graph import ABCPropertyGraph
+from ..slivers.capacities_labels import Capacities, Labels
 
 
 class ElementType(enum.Enum):
@@ -40,6 +41,7 @@ class ModelElement(ABC):
     """
     Abstract element of a model
     """
+
     @abstractmethod
     def __init__(self, *, name: str, node_id: str, topo: Any):
         """
@@ -88,9 +90,11 @@ class ModelElement(ABC):
 
     def __repr__(self):
         labels, node_props = self.topo.graph_model.get_node_properties(node_id=self.node_id)
+        # filter out some properties we don't need
         node_props.pop(ABCPropertyGraph.GRAPH_ID)
         if str(self.topo.__class__) == "<class 'fim.user.topology.ExperimentTopology'>":
             node_props.pop(ABCPropertyGraph.NODE_ID)
+            node_props.pop(ABCPropertyGraph.PROP_STITCH_NODE)
         ret = str(labels) + str(node_props)
         return ret
 
@@ -120,4 +124,50 @@ class ModelElement(ABC):
         Set multiple properties of a model element
         :param kwargs:
         :return:
-        """
\ No newline at end of file
+        """
+
+    @property
+    def name(self):
+        return self._name
+
+    @name.setter
+    def name(self, value: str):
+        self._name = value
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('name', value)
+
+    @property
+    def capacities(self):
+        return self.get_property('capacities') if self.__dict__.get('topo', None) is not None else None
+
+    @capacities.setter
+    def capacities(self, value: Capacities):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('capacities', value)
+
+    @property
+    def labels(self):
+        return self.get_property('labels') if self.__dict__.get('topo', None) is not None else None
+
+    @labels.setter
+    def labels(self, value: Labels):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('labels', value)
+
+    @property
+    def details(self):
+        return self.get_property('details') if self.__dict__.get('topo', None) is not None else None
+
+    @details.setter
+    def details(self, value: str):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('details', value)
+
+    @property
+    def node_map(self):
+        return self.get_property('node_map') if self.__dict__.get('topo', None) is not None else None
+
+    @node_map.setter
+    def node_map(self, value):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('node_map', value)
\ No newline at end of file
